package mark.core.io;
import java.io.*;

/**
 * A FileFilter that looks for a pattern within the canonical
 * filename.  In the pattern, use '/' as the pathname separator: the filter
 * will convert automatically.
 *
 * @author Mark Chavira
 */

public class NamePatternFileFilter implements FileFilter {

  private java.util.regex.Pattern fPattern;

  /**
   * Initializes the filter.
   *
   * @param pattern the pattern for which to look.
   */
   
  public NamePatternFileFilter (String pattern) {
    fPattern = java.util.regex.Pattern.compile (pattern);
  }

  /**
   * @see FileFilter
   */
    
  public boolean accept (File f) {
    try {
      return
        fPattern.matcher (
          f.getCanonicalPath ().
            replace (File.pathSeparatorChar, '/')).find ();
    } catch (Exception e) {
      return false;
    }
  }

}
