package mark.core.rdag;

/**
 * An interface for an object that supplies information needed to write an
 * rdag as an omnigraffle script.
 * 
 * @author chavira
 */

public interface ScriptInformation {

  /**
   * Possible Edge styles.
   * 
   * @author chavira
   */
  
  public enum LineStyle {
    NORMAL (0),
    DASHED (1);
    private int fOmniId;
    LineStyle (int omniId) {fOmniId = omniId;}
    public int omniId () {return fOmniId;}
  }

  /**
   * Returns the string that defines the properties for the make new shape
   * command executed when drawing the given node.
   * 
   * @param n the given node.
   * @return the string.
   */
  
  public String nodeProperties (Node n) throws Exception;
  
  /**
   * If the ith outgoing edge of the given node has a label, returns it.
   * Otherwise, returns "".
   *
   * @param n the given node.
   * @param i identifies the edge.
   * @return the label.
   */
  
  public String ithOutgoingEdgeLabel (Node n, int i) throws Exception;

  /**
   * Returns the style of the ith outgoing edge of the given node.  If
   * combining edges, then the style of a combined edge is the style of its
   * lowest numbered edge.
   *
   * @param n the given node.
   * @param i identifies the edge.
   * @return the style.
   */

  public LineStyle ithOutgoingEdgeStyle (Node n, int i) throws Exception;
  
  /**
   * Returns whether to combine outgoing when they both exit the given node
   * and enter the same node.
   *
   * @param n the given node.
   * @return whether to combine the edges.
   */
  
  public boolean combineOutgoingEdges (Node n) throws Exception;
  
  /**
   * Returns whether or not to draw the outgoing edge labels for the given
   * node.
   *
   * @return whether to draw the edge lables.
   */
  
  public boolean drawOutgoingEdgeLabels (Node n) throws Exception;

}
