package mark.core.util;
import java.util.*;

/**
 * An unmodifiable list that is also a set.  The collection supports null
 * elements, does not allow multiple copies of an element, and can map from
 * element to index in constant time.
 * <p>
 * This class conforms to the immutable design pattern.
 *
 * @author Mark Chavira
 */

public class OrderedSet<E>
 extends AbstractList<E> implements Set<E> {

  private E[] fIntToObject;
  private HashMap<E,Integer> fObjectToInt;
  
  /**
   * @see AbstractCollection
   */
  
  private void myAdd (E e) {
    if (fObjectToInt.containsKey (e)) {
      throw new Error ("Duplicate element : " + e);
    }
    int index = fObjectToInt.size ();
    fObjectToInt.put (e, new java.lang.Integer (index));
    fIntToObject[index] = e;
  }
  
  /**
   * @see AbstractCollection
   */
  
  public E get (int index) {
    if (index < 0 || index >= size ()) {
      throw new IndexOutOfBoundsException ();
    }
    return fIntToObject[index];
  }
  
  /**
   * @see Collection
   */
  
  public int size () {
    return fIntToObject.length;
  }

  /**
   * @see Collection
   */
  
  public boolean contains (Object o) {
    return fObjectToInt.containsKey (o);
  }

  /**
   * @see AbstractList
   */

  public int indexOf (Object o) {
    java.lang.Integer ans = fObjectToInt.get (o);
    return (ans == null) ? -1 : ans;
  }

  /**
   * @see AbstractList
   */
   
  public int lastIndexOf (Object o) {
    return indexOf (o);
  }

  /**
   * Initializes the map and associates the given objects.
   *
   * @param objects the objects to associate.
   */

  public OrderedSet (E... objects) {
    fIntToObject = (E[])new Object[objects.length];
    fObjectToInt = new HashMap<E,Integer> (objects.length);
    for (int i = 0; i < objects.length; i++) {myAdd (objects[i]);}
  }

  /**
   * Initializes the map and associates the given objects.
   *
   * @param objects the objects to associate.
   */
  
  public OrderedSet (Collection<? extends E> c) {
    fIntToObject = (E[])new Object[c.size ()];
    fObjectToInt = new HashMap<E,Integer> (c.size ());
    for (E e : c) {myAdd (e);}
  }
  
}
