package mark.inf.eo;

/**
 * An EliminationOrderEngine that uses minsize.
 *
 * @author Mark Chavira
 */

public class EoeMinsize extends Eoe {

  /**
   * The pq element.
   * 
   * @author Mark Chavira
   */
  
  private static class Cost implements Comparable {
    public double logClusterSize;
    public int compareTo (Object o) {
      Cost c = (Cost)o;
      return
        mark.core.math.Util.close (logClusterSize, c.logClusterSize) ? 0 :
        logClusterSize < c.logClusterSize ? -1 :
        +1;
    }
  }

  /**
   * @see EliminationOrderAlgorithm
   */

  protected Comparable bruteCost (
   int[][] nodeToAdj, int[] nodeToAdjSize, int n, boolean[] marked) {
    Cost ans = new Cost ();
    ans.logClusterSize = fNodeToLogCardinality[n];
    for (int j = 0; j < nodeToAdjSize[n]; j++) {
      int oneHop = nodeToAdj[n][j];
      ans.logClusterSize += fNodeToLogCardinality[oneHop];
    }
    return ans;
  }

  /**
   * @see EliminationOrderAlgorithm
   */
                
  protected void updateCosts (
   int[][] nodeToAdj, int[] nodeToAdjSize, int removed,
   boolean[] marked, boolean[][] marked2,
   mark.core.util.IntKeyPQ pq) throws Exception {
    for (
     int oneHopIndex1 = 0; oneHopIndex1 < nodeToAdjSize[removed];
     oneHopIndex1++) {
      int oneHop1 = nodeToAdj[removed][oneHopIndex1];
      if (pq.contains (oneHop1)) {
        ((Cost)pq.getKey (oneHop1)).logClusterSize -=
          fNodeToLogCardinality[removed];
        pq.changeKey (oneHop1);
      }
      for (
       int oneHopIndex2 = oneHopIndex1 + 1;
       oneHopIndex2 < nodeToAdjSize[removed]; oneHopIndex2++) {
        int oneHop2 = nodeToAdj[removed][oneHopIndex2];
        if (!marked2[oneHopIndex1][oneHop2]) {
          if (pq.contains (oneHop1)) {
            ((Cost)pq.getKey (oneHop1)).logClusterSize +=
              fNodeToLogCardinality[oneHop2];
            pq.changeKey (oneHop1);
          }
          if (pq.contains (oneHop2)) {
            ((Cost)pq.getKey (oneHop2)).logClusterSize +=
              fNodeToLogCardinality[oneHop1];
            pq.changeKey (oneHop2);
          }
        }
      }
    }
  }

}
