package mark.inf.eo;

/**
 * An interface for an object that can evaluate the quality of an elimination
 * prefix by assigning it a score that can then be used to compare with other
 * prefixes.
 * 
 * @author Mark Chavira
 */

public interface ScoringStrategy {

  /**
   * Initializes the computations.
   */
  
  public Comparable startOverWithNewPrefix (int[] prefix, double log2MaxCluster);
  
  /**
   * Returns the score of the given prefix, which has one additional variable
   * appended to the last seen prefix.
   * 
   * @param prefix the given prefix.
   * @param prefixLength the length of the given prefix.
   * @param log2MaxClusterSize the given log_2 of maximum cluster size.
   * @return the score or null if this prefix should be rejected.
   */
  
  public Comparable addVar (
   int[] prefix, int prefixLength, double log2MaxCluster);
  
  /**
   * Returns the score of combining all remaining nodes.
   * 
   * @param log2MaxClusterSize the given log_2 of maximum cluster size.
   */

  public Comparable combineAtEnd (double log2MaxCluster);

  /**
   * Resets the strategy to its state after it was last initialized.
   */
    
  public void revertToPrefix ();
    
}
