package mark.inf.gate;

/**
 * An enumerated type describing logic gate health states.
 *
 * @author Mark Chavira
 */
 
public enum GateHealth {
  
  /**
   * A constant for the health set {healthy}.  For this health set, there is
   * no health variable.
   */
   
  ALWAYS_HEALTHY (0, -1, -1),

  /**
   * A constant for the health set {healthy, stuck-at-zero}.  For this health
   * set, a health value of 0 means healthy and 1 means stuck-at-zero.
   */

  STUCK_ZERO (0, 1, -1, Gate.HEALTHY_STRING, Gate.STUCK_AT_ZERO_STRING),

  /**
   * A constant for the health set {healthy, stuck-at-one}.  For this health
   * set, a health value of 0 means healthy and 1 means stuck-at-one.
   */
   
  STUCK_ONE (0, -1, 1, Gate.HEALTHY_STRING, Gate.STUCK_AT_ONE_STRING),

  /**
   * A constant for the health set {healthy, stuck-at-zero, stuck-at-one}.
   * For this health set, a health value of 0 means healthy, 1 means
   * stuck-at-zero, and 2 means-stuck-at one.
   */

  STUCK_BOTH (
    0, 1, 2, Gate.HEALTHY_STRING, Gate.STUCK_AT_ZERO_STRING,
    Gate.STUCK_AT_ONE_STRING);

  /**
   * The index of the healthy state.
   */
  
  private int fHealthyIndex;
  
  /**
   * The index of the stuck at zero state or -1 if none.
   */
  
  private int fStuckZeroIndex;
  
  /**
   * The index of the stuck at one state or -1 if none.
   */
  
  private int fStuckOneIndex;
  
  /**
   * The domain.
   */
  
  private mark.core.util.OrderedSet fHealthDomain;
  
  /**
   * The constructor.
   * 
   * @param domain the domain.
   */
  
  private GateHealth (
   int healthyIndex, int stuckZeroIndex, int stuckOneIndex,
   String... domain) {
    fHealthyIndex = healthyIndex;
    fStuckZeroIndex = stuckZeroIndex;
    fStuckOneIndex = stuckOneIndex;
    fHealthDomain =
      (domain.length == 0) ? null :
      new mark.core.util.OrderedSet ((Object[])domain);
  }

  /**
   * Returns the names of the health states associated with the health type.
   *
   * @return the names of the health states.
   */

  public mark.core.util.OrderedSet domain () {
    return fHealthDomain;
  }

  /**
   * Returns the index of the healthy state.
   * 
   * @return the index.
   */
  
  public int healthyIndex () {
    return fHealthyIndex;
  }

  /**
   * Returns the index of the stuck at zero state or -1 if none.
   * 
   * @return the index.
   */
  
  public int stuckZeroIndex () {
    return fStuckZeroIndex;
  }
  /**
   * Returns the index of the stuck at one state or -1 if none.
   * 
   * @return the index.
   */
  
  public int stuckOneIndex () {
    return fStuckOneIndex;
  }
  
}
