package mark.reason.core;
import java.util.*;

/**
 * A mark.inf.core.SingleIterator that also iterates over map
 * instantiations.
 * <xmp>
 * SingleIterator si = new SingleIterator (L, E);
 * for (int i = 0; i < numCompatibleInstantiations (); i++) {
 *   int p = si.next ();
 *   // p is the current int instantiation
 *   // si.t is the current int[] instantiation
 *   // si.w is the current map instantiation
 * }
 * </xmp>
 * <p>
 * Currently, this class does not support evidence as does SingleIterator.
 * I just haven't had a need for it yet.
 *
 * @author Mark Chavira
 */

public class SingleIterator extends mark.inf.core.SingleIterator {

  private Var[] fVariables;

  /**
   * The current world.  This object is provided for efficiency and should be
   * considered read-only, even thought java does not enforce this
   * restriction.
   */

  public HashMap<Var,Double> w;
  
  /**
   * Initializes the iterator.
   *
   * @param L the variable list.
   * @param E the evidence.
   */
  
  public SingleIterator (Var[] L, Map E) {
    super (
      Instantiation.cardinalityArray (L),
      E == null ? null : Instantiation.convertEvidenceToIntArray (L, E));
    fVariables = L;
    w = Instantiation.convertEvidenceToMap (fVariables, k);
  }
    
  /**
   * Initializes the iterator.
   *
   * @param L the variable list.
   * @param E the evidence.
   */
  
  public SingleIterator (Collection L, Map E) {
    this ((Var[])L.toArray (new Var[L.size ()]), E);
  }
  
  /**
   * Returns whether there is another instantation.
   *
   * @return whether there is another instantation.
   */
  
  public int next () {
    int ans = super.next ();
    for (int i = fFirst; i >= 0; i--) {
      if (fInc[i]) {
        w.put (fVariables[i], fVariables[i].minDomainElement () + k[i]);
        if (k[i] != 0) {
          break;
        }
      }
    }
    return ans;
  }
  
}
