package mark.reason.flt;
import java.util.*;

/**
 * A filter that combines constants into a single constant.
 * <p>
 * This class participates in a cliObject design pattern.  This class accepts
 * no cli parametrs.
 * <p>
 * This class conforms to the immutable design pattern.
 * <p>
 * This class conforms to the singleton design pattern.
 * 
 * @author chavira
 */

public enum FltCombineConstants implements Flt, mark.core.shell.CliObject {

  /**
   * The singleton instance.
   */
  
  INSTANCE;
  
  /**
   * Combines any constants into a single constant.
   * 
   * @param pots the potentials.
   * @return the new potentials.
   */
  
  public static HashSet<mark.reason.net.Pot> combine (
   Set<mark.reason.net.Pot> pots) throws Exception {
    HashSet<mark.reason.net.Pot> ans =
      new HashSet<mark.reason.net.Pot> (pots);
    HashSet<mark.reason.net.Pot> constants = FltConstants.constants (pots);
    if (constants.size () == 0) {return ans;}
    ans.removeAll (constants);
    mark.reason.net.Pot constant = FltMultiply.multiply (constants, null);
    if (constant.getCp (0) != 1.0) {ans.add (constant);}
    return ans;
  }
  
  /**
   * @see Filter
   */
  
  public HashSet<mark.reason.net.Pot> filter (
   Random r, Set<mark.reason.net.Pot> pots) throws Exception {
    return combine (pots);
  }

}
