package mark.reason.flt;
import java.util.*;

/**
 * A filter that multiplies potentials together.
 * <p>
 * This class participates in a cliObject design pattern.  This class accepts
 * no cli parameters.
 *
 * @author Mark Chavira
 */

public enum FltMultiply implements Flt, mark.core.shell.CliObject {

  /**
   * The singleton instance.
   */
  
  INSTANCE;

  /**
   * Multiplies together the given potentials.
   *
   * @param pots the given potentials.
   * @param fact the potential factory, which may be null if pots is
   *   not empty.
   * @return the product.
   */

  public static mark.reason.net.Pot multiply (
   Set<mark.reason.net.Pot> pots, mark.reason.net.PotFact fact)
   throws Exception {
    if (pots.isEmpty ()) {
      return fact.create (1.0);
    }
    HashSet<mark.reason.net.Pot> s = new HashSet<mark.reason.net.Pot> (pots);
    mark.reason.net.Pot d = s.iterator ().next ();
    s.remove (d);
    return d.multiply (s);
  }

  /**
   * @see Filter
   */

  public HashSet<mark.reason.net.Pot> filter (
   Random r, Set<mark.reason.net.Pot> pots) throws Exception {
    if (pots.size () == 0) {
      throw new Exception ("Cannot multiply empty set of potentials!");
    }
    return new HashSet<mark.reason.net.Pot> (
      Collections.singleton (multiply (pots, null)));
  }

}
