package mark.reason.flt;
import java.util.*;
import mark.reason.core.*;

/**
 * A filter that shrinks the given potentials according to the given
 * evidence.
 * <p>
 * This class participates in a cliObject (mark.reason.bn.Filter, this)
 * design pattern.  This class specifies the following cli parameters:
 * <ol>
 * <li> eg : the evidence generator.
 * </ol>
 *
 * @author Mark Chavira
 */

public class FltShrink implements Flt, mark.core.shell.CliObject {

  /**
   * The evidence generator.
   */
  
  private mark.reason.query.Eg fEg;

  /**
   * Shrinks the given potentials according to the given evidence.
   *
   * @param pots the given potentials.
   * @param evidence the given evidence.
   * @param oldToNew if specified, fills with a map from old to new
   *   potential.
   * @return the new potentials.
   */
  
  public static HashSet<mark.reason.net.Pot> shrink (
   Set<mark.reason.net.Pot> pots, Map<Var,Double> evidence,
   Map<mark.reason.net.Pot,mark.reason.net.Pot> oldToNew)
   throws Exception {
    HashSet<mark.reason.net.Pot> ans = new HashSet<mark.reason.net.Pot> ();
    for (mark.reason.net.Pot p : pots) {
      mark.reason.net.Pot newPot = p.shrink (evidence);
      if (oldToNew != null) {
        oldToNew.put (p, newPot);
      }
      ans.add (newPot);
    }
    return ans;
  }

  /**
   * Shrinks the given potentials according to the given evidence.
   *
   * @param pots the given potentials.
   * @param evidence the given evidence.
   * @return the new potentials.
   */
  
  public static HashSet<mark.reason.net.Pot> shrink (
   Set<mark.reason.net.Pot> pots, Map<Var,Double> evidence)
   throws Exception {
    return shrink (pots, evidence, null);
  }

  /**
   * The constructor.
   *
   * @param qvs the variable selector to use to identify query variables.
   */
  
  public FltShrink (mark.reason.query.Eg eg) {
    fEg = eg;
  }

  /**
   * @see CliObject
   */

  public static Object create (Object[] mandated, Object[] cli)
   throws Exception {
    if (mandated.length != 0 || cli.length != 1) {
      throw new Exception ("Incorrect parameters!");
    }
    return
      new FltShrink (
        (mark.reason.query.Eg)
        mark.core.shell.CliUtil.commandLineObject2 (cli[0]));
  }

  /**
   * @see Filter
   */
  
  public HashSet<mark.reason.net.Pot> filter (
   Random r, Set<mark.reason.net.Pot> pots) throws Exception {
    return
      shrink (
        pots, fEg.generate (r, pots, CoreUtil.variables (pots), 1).get (0));
  }

}
