package mark.reason.net;

/**
 * An interface for an object that constructs potentials.
 * <p>
 * This class participates in a factory design pattern:
 * <ul>
 * <li> PotentialFactory fills the creator role.
 * <li> Potential fills the product role.
 * <li> Various classes fill the concrete creator role.
 * <li> Various classes fill the concrete product role.
 * </ul>
 * <p>
 * This class participates in a command-line-object pattern:
 * <ul>
 * <li> This class fills the base role.
 * <li> Various classes fill the concrete class role.
 * </ul>
 * <p>
 * This class specifies the following mandated parameters:
 * <ol>
 * <li> r : a random generator.
 * <li> variables : an ordered set of variables.
 * </ol>
 *
 * @author Mark Chavira
 */

public abstract interface PotFact {

  /**
   * Returns a potential denoting the same mapping as the given
   * potential.
   * <p>
   * The given potential need not have been produced by this factory.
   *
   * @param d the given potential.
   */
  
  public Pot create (Pot d) throws Exception;

  /**
   * Creates a potential denoting a constant mapping.
   *
   * @param d the constant.
   */
  
  public Pot create (double d) throws Exception;

}
