package mark.reason.net;
import java.util.*;
import java.io.*;


/**
 * An interface for an object that produces a set of potentials from a
 * reader.
 * <p>
 * This class participates in a command-line-object pattern:
 * <ul>
 * <li> This class fills the base role.
 * <li> Various classes fill the concrete class role.
 * </ul>
 * This class specifies no mandated parameters.
 *
 * @author Mark Chavira
 */

public interface Pr {
  
  /**
   * Parses the text in the given reader, closes the given reader, and
   * returns the resulting set of potentials.
   *
   * @param r the given reader.
   * @return the resulting potentials.
   */
  
  HashSet<Pot> read (Reader r) throws Exception;
  
  /**
   * Parses the text in the given file and returns the set of potentials.
   * For most implementations, simpley calls read () with a FileReader.
   * However, in some cases, it may be advantageous to deal with a file in
   * another way.  For example, see PrSmile.
   * 
   * @param f the given file.
   * @return the resulting potentials.
   */
  
  HashSet<Pot> read (java.io.File f) throws Exception;
  
}
