package mark.reason.plg;
import java.util.*;

/**
 * An and node in a plg.
 * <p>
 * This class conforms to the immutable design pattern.
 *
 * @author Mark Chavira
 */

public class NodeAnd extends NodeMultipleChild {

  //=========================================================================
  // Constructors:
  //=========================================================================

  /**
   * Initializes the node.
   *
   * @param children the node's children.
   */
  
  public NodeAnd (Node... children) {
    super (children);
  }
  
  /**
   * Initializes the node.
   *
   * @param children the node's children.
   */
  
  public NodeAnd (Collection children) {
    super (children);
  }
  
  //=========================================================================
  // Node Methods:
  //=========================================================================

  /**
   * @see mark.core.plg.Node
   */

  public String label () {
    return "^";
  }

  /**
   * @see Node
   */

  public Node clone (Node[] children) {
    return new NodeAnd (children);
  }
 
  //=========================================================================
  // Fulfillments:
  //=========================================================================

  /**
   * @see Node.
   */

  protected boolean modelTestHelper (boolean[] ch, Map w) {
    for (int i = 0; i < ch.length; i++) {
      if (!ch[i]) {
        return false;
      }
    }
    return true;
  }

  /**
   * @see Node
   */

  protected Node reduceHelper (Node[] ch) {
    ArrayList childList = new ArrayList ();
    for (int i = 0; i < ch.length; i++) {
      if (ch[i] instanceof NodeFalse) {
        return NodeFalse.INSTANCE;
      } else if (ch[i].getClass () == getClass ()) {
        for (int j = 0; j < ch[i].numOutgoingEdges (); j++) {
          childList.add (ch[i].nthChild (j));
        }
      } else if (!(ch[i] instanceof NodeTrue)) {
        childList.add (ch[i]);
      }
    }
    if (childList.isEmpty ()) {
      return NodeTrue.INSTANCE;
    }
    if (childList.size () == 1) {
      return (Node)childList.get (0);
    }
    return clone (toNodeArray (childList));
  }
   
  /**
   * @see Node
   */
   
  protected Node simplifyConnectivesHelper (Node[] ch) {
    if (ch.length == 0) {
      return NodeTrue.INSTANCE;
    }
    if (ch.length == 1) {
      return ch[0];
    }
    Node[] twoChildren = new Node[2];
    for (int i = ch.length - 2; i >= 0; i--) {
      twoChildren[0] = ch[i];
      twoChildren[1] = ch[i + 1];
      ch[i] = new NodeAnd (twoChildren);
    }
    return ch[0];
  }

  /**
   * @see Node
   */

  protected Node notAndOrPlgHelper (Node[] ch) throws Exception {
    return new NodeAnd (ch);
  }

}
