package mark.reason.plg;
import java.util.*;
import mark.reason.core.*;

/**
 * A base class for an atom in a plg.
 * <p>
 * This class conforms to the immutable design pattern.
 *
 * @author Mark Chavira
 */

public abstract class NodeAtom extends NodeLeaf {

  //=========================================================================
  // Fields:
  //=========================================================================

  private Var fVariable;

  //=========================================================================
  // Constructors:
  //=========================================================================

  /**
   * Initializes the node.
   *
   * @param v the node's variable.
   */
  
  protected NodeAtom (Var v) {
    fVariable = v;
  }
  
  //=========================================================================
  // Object Methods:
  //=========================================================================
  
  /**
    * @see java.lang.Object
   */
  
  public boolean equals (Object o) {
    if (!(o instanceof NodeAtom)) {
      return false;
    }
    NodeAtom a = (NodeAtom)o;
    return variable () == a.variable () && values ().equals (a.values ());
  }
  
  /**
   * @see Node
   */
  
  protected int computeHash () {
    return variable ().hashCode () + values ().hashCode ();
  }

  //=========================================================================
  // Node Methods:
  //=========================================================================

  /**
   * Returns the node's variable.
   *
   * @return the variable.
   */
  
  public Var variable () {
    return fVariable;
  }
  
  /**
   * Returns the values that the atom assigns the variable.
   *
   * @return the value.
   */

  public abstract Object values ();

  /**
   * Returns the integer value that the atom assigns the variable or throws
   * an exception if the node assigns more than one value or a value that is
   * not an integer.
   *
   * @return the value.
   */

  public int intValue () throws Exception {
    throw new Exception ("Cannot call intValue () on this type of atom!");
  }
  
  //=========================================================================
  // Graph Methods:
  //=========================================================================

  /**
   * @see Node
   */

  protected Node conditionHelper (Node[] ch, Map pw) {
    if (pw.get (variable ()) == null) {
      return this;
    }
    return
      modelTestHelper (null, pw) ?
      ((Node)NodeTrue.INSTANCE) :
      ((Node)NodeFalse.INSTANCE);
  }

}
