package mark.reason.plg;
import java.util.*;
import mark.reason.core.*;

/**
 * A binary atom in a plg.
 * <p>
 * This class conforms to the immutable design pattern.
 *
 * @author Mark Chavira
 */

public class NodeAtomBinary extends NodeAtom {

  private static final Boolean VALUE = true;
  
  //=========================================================================
  // Constructors:
  //=========================================================================

  /**
   * Initializes the node.
   *
   * @param v the node's variable.
   */
  
  public NodeAtomBinary (VarBinary v) {
    super (v);
  }
  
  //=========================================================================
  // Node Methods:
  //=========================================================================

  /**
   * @see mark.core.plg.Node
   */

  public String label () {
    return variable ().name ();
  }

  /**
   * @see mark.core.plg.NodeAtom
   */
  
  public Object values () {
    return VALUE;
  }
  
  /**
   * @see mark.core.plg.NodeAtom
   */

  public int intValue () throws Exception {
    return 1;
  }

  //=========================================================================
  // Graph Methods:
  //=========================================================================

  /**
   * @see .Theory
   */

  protected boolean modelTestHelper (
   boolean[] ch, Map<Var,Double> w) {
    return w.get (variable ()) == 1.0;
  }

  /**
   * @see Node
   */
  
  protected Node renameHelper (Node[] ch, Map v) {
    return new NodeAtomBinary ((VarBinary)v.get (variable ()));
  }

  /**
   * @see Node
   */

  protected Node reduceHelper (Node[] ch) {
    return this;
  }

}
