package mark.reason.plg;
import java.util.*;

/**
 * An equivalent node in a plg.
 * <p>
 * This class conforms to the immutable design pattern.
 *
 * @author Mark Chavira
 */

public class NodeEquivalent extends NodeMultipleChild {

  //=========================================================================
  // Constructors:
  //=========================================================================

  /**
   * Initializes the node.
   *
   * @param lhs the first of two children.
   * @param rhs the second of two children.
   */
  
  public NodeEquivalent (Node lhs, Node rhs) {
    super (new Node[] {lhs, rhs});
  }

  /**
   * Initializes the node.
   *
   * @param children the node's children.
   */
  
  public NodeEquivalent (Node... children) {
    super (children);
  }
  
  /**
   * Initializes the node.
   *
   * @param children the node's children.
   */
  
  public NodeEquivalent (Collection children) {
    super (children);
  }
  
  //=========================================================================
  // Node Methods:
  //=========================================================================

  /**
   * @see mark.core.plg.Node
   */

  public String label () {
    return "<->";
  }

  /**
   * @see Node
   */

  public Node clone (Node[] children) {
    return new NodeEquivalent (children);
  }
 
  //=========================================================================
  // Fulfillments:
  //=========================================================================

  /**
   * @see Theory
   */

  protected boolean modelTestHelper (boolean[] ch, Map w) {
    for (int i = 1; i < ch.length; i++) {
      if (ch[i] != ch[i - 1]) {
        return false;
      }
    }
    return true;
  }

  /**
   * @see Node
   */

  protected Node reduceHelper (Node[] ch) {
    return clone (ch); // slashes: not implemented yet
  }

  /**
   * @see Node
   */
  
  protected Node simplifyConnectivesHelper (Node[] ch) {
    if (numOutgoingEdges () < 2) {
      return NodeTrue.INSTANCE;
    }
    Node[] newCh = new Node[numOutgoingEdges () - 1];
    for (int i = 0; i < newCh.length; i++) {
      newCh[i] = new NodeEquivalent (new Node[] {ch[0], ch[i]});
    }
    return new NodeAnd (newCh);
  }

  /**
   * @see Node
   */

  protected Node notAndOrPlgHelper (Node[] ch) throws Exception {
    Node[] ch2 = (Node[])ch.clone ();
    for (int i = 0; i < ch2.length; i++) {
      ch2[i] = new NodeNot (ch2[i]);
    }
    return new NodeOr (new Node[] {new NodeAnd (ch), new NodeAnd (ch2)});
  }

}
