package mark.reason.plg;
import java.util.*;

/**
 * A false node in a plg.
 * <p>
 * This class conforms to the immutable design pattern.
 *
 * This class conforms to the singleton design pattern.
 *
 * @author Mark Chavira
 */

public class NodeFalse extends NodeLeaf implements mark.core.util.Singleton {

  private NodeFalse () {}

  /**
   * The singleton instance.
   */

  public static NodeFalse INSTANCE = new NodeFalse ();

  //=========================================================================
  // Node Methods:
  //=========================================================================

  /**
   * @see Node
   */
  
  protected int computeHash () {
    return Boolean.FALSE.hashCode ();
  }
  
  /**
   * @see mark.core.plg.Node
   */

  public String label () {
    return "false";
  }

  //=========================================================================
  // Graph Methods:
  //=========================================================================

  /**
   * @see Node
   */

  protected boolean modelTestHelper (boolean[] ch, Map w) {
    return false;
  }

  /**
   * @see Node
   */

  protected Node conditionHelper (Node[] ch, Map pw) {
    return this;
  }

  /**
   * @see Node
   */

  protected Node renameHelper (Node[] ch, Map v) {
    return this;
  }

  /**
   * @see Node
   */

  protected Node reduceHelper (Node[] ch) {
    return this;
  }

}
