package mark.reason.plg;
import java.util.*;

/**
 * An internal node in a plg.
 * <p>
 * This class conforms to the immutable design pattern.
 *
 * @author Mark Chavira
 */

public abstract class NodeInternal extends Node {

  /**
   * @see java.lang.Object
   */

  public boolean equals (Object o) {
    if (getClass () != o.getClass ()) {
      return false;
    }
    NodeInternal n = (NodeInternal)o;
    if (numOutgoingEdges () != n.numOutgoingEdges ()) {
      return false;
    }
    for (int i = 0; i < numOutgoingEdges (); i++) {
      if (nthChild (i) != n.nthChild (i))  {
        return false;
      }
    }
    return true;
  }
  
  /**
   * @see Node
   */
  
  protected int computeHash () {
    int ans = getClass ().hashCode ();
    for (int i = 0; i < numOutgoingEdges (); i++) {
      ans += nthChild (i).hashCode ();
    }
    return ans;
  }

  /**
   * @see mark.core.rdag.Node
   */
  
  public mark.core.rdag.Node nthChild (int n) {
    return nthCastedChild (n);
  }

  /**
   * @see Node
   */

  protected Node renameHelper (Node[] ch, Map v) {
    return clone (ch);
  }

}
