package mark.reason.plg;
import java.util.*;

/**
 * A base class for an AND or OR node in a plg.
 * <p>
 * This class conforms to the immutable design pattern.
 *
 * @author Mark Chavira
 */

public abstract class NodeMultipleChild extends NodeInternal {

  //=========================================================================
  // Fields:
  //=========================================================================

  private Node[] fChildren;

  //=========================================================================
  // Constructors:
  //=========================================================================

  /**
   * Initializes the node.
   *
   * @param children the node's children.
   */
  
  protected NodeMultipleChild (Node... children) {
    fChildren = (Node[])children.clone ();
  }
  
  /**
   * Initializes the node.
   *
   * @param children the node's children.
   */
  
  protected NodeMultipleChild (
   Collection<mark.core.rdag.Node> children) {
    fChildren = new Node[children.size ()];
    int index = 0;
    for (mark.core.rdag.Node n : children) {
      fChildren[index++] = (Node)n;
    }
  }
  
  //=========================================================================
  // Node Methods:
  //=========================================================================

  /**
   * @see mark.core.plg.Node
   */

  public int numOutgoingEdges () {
    return fChildren.length;
  }

  /**
   * @see Node
   */
  
  public Node nthCastedChild (int n) {
    return fChildren[n];
  }

  //=========================================================================
  // Graph Methods:
  //=========================================================================

  /**
   * @see Node
   */

  protected Node conditionHelper (Node[] ch, Map pw) {
    return clone (ch);
  }

  /**
   * @see Node
   */

  public String toPrefixString () {
    StringBuilder buf = new StringBuilder ();
    buf.append ("(");
    buf.append (label ());
    for (int i = 0; i < numOutgoingEdges (); i++) {
      buf.append (" ");
      buf.append (nthCastedChild (i).toPrefixString ());
    }
    buf.append (")");
    return buf.toString ();
  }

  /**
   * @see Node
   */

  public String toInfixString () {
    StringBuilder buf = new StringBuilder ();
    buf.append ("(");
    for (int i = 0; i < numOutgoingEdges (); i++) {
      buf.append (nthCastedChild (i).toInfixString ());
      if (i != numOutgoingEdges () - 1) {
        buf.append (label ());
      }
    }
    buf.append (")");
    return buf.toString ();
  }

}
