package mark.reason.plg;
import java.util.*;

/**
 * A not node in a plg.
 * <p>
 * This class conforms to the immutable design pattern.
 *
 * @author Mark Chavira
 */

public class NodeNot extends NodeInternal {

  //=========================================================================
  // Fields:
  //=========================================================================

  private Node fChild;

  //=========================================================================
  // Constructors:
  //=========================================================================

  /**
   * Initializes the node.
   *
   * @param children the node's children.
   */
  
  public NodeNot (Node child) {
    fChild = child;
  }
  
  //=========================================================================
  // Node Methods:
  //=========================================================================

  /**
   * @see mark.core.plg.Node
   */

  public String label () {
    return "~";
  }

  /**
   * @see mark.core.plg.Node
   */

  public int numOutgoingEdges () {
    return 1;
  }

  /**
   * @see Node
   */
  
  public Node nthCastedChild (int n) {
    return fChild;
  }

  /**
   * Returns the node's child.
   *
   * @return the node's child.
   */
  
  public Node child () {
    return fChild;
  }

  //=========================================================================
  // Graph Methods:
  //=========================================================================

  /**
   * @see .Theory
   */

  protected boolean modelTestHelper (boolean[] ch, Map w) {
    return !ch[0];
  }

  /**
   * @see Node
   */

  protected Node conditionHelper (Node[] ch, Map pw) {
    return new NodeNot (ch[0]);
  }

  /**
   * @see Node
   */

  public String toPrefixString () {
    return "(~" + child ().toPrefixString () + ")";
  }

  /**
   * @see Node
   */

  public String toInfixString () {
    return "(" + label () + " " + child ().toInfixString () + ")";
  }

  /**
   * @see Node
   */
   
  public Node clone (Node[] children) {
    return new NodeNot (children[0]);
  }

  /**
   * @see Node
   */

  protected Node reduceHelper (Node[] ch) {
    if (ch[0] instanceof NodeNot) {
      return ch[0].nthCastedChild (0);
    }
    if (ch[0] instanceof NodeFalse) {
      return NodeTrue.INSTANCE;
    }
    if (ch[0] instanceof NodeTrue) {
      return NodeFalse.INSTANCE;
    }
    return new NodeNot (ch[0]);
  }

  /**
   * @see Node
   */
   
  protected Node simplifyConnectivesHelper (Node[] ch) {
    return new NodeNot (ch[0]);
  }

  /**
   * @see Node
   */

  protected Node notAndOrPlgHelper (Node[] ch) throws Exception {
    return new NodeNot (ch[0]);
  }

}
