package mark.reason.struct;
import java.util.*;
import mark.reason.core.*;

/**
 * An interface for an object that inputs a network and a cluster limit,
 * produces an approximate network by cloning nodes in the given network
 * until the maximum cluster is less than or equal to the given limit, and
 * returns the approximate network and a map from variable in the given
 * network to set of its clones in the approximate network.  Introduced clone
 * roots are indicator tables (all parameters set to one).
 * 
 * @author Mark Chavira
 */

public interface SplittingStrategy {

  /**
   * The result returned by the strategy.
   * 
   * @author Mark Chavira
   */
  
  public static class Result<T extends Scope> {
    
    /**
     * The approximate network.
     */
    
    public HashSet<T> fScopes;
    
    /**
     * The log of the maximum cluster size achieved.
     */
    
    public double fLog2MaxClusterAchieved;
    
  }

  /**
   * Produces a network that approximates the given network and has maximum
   * cluster <= the given limit as described in the class header.
   * 
   * @param r a random generator.
   * @param originalScopes the given network.
   * @param originalVarToClones on entry, an empty map; on exit, the map from
   *   variable in the given network to set of corresponding variables in the
   *   approximate network.
   * @param logClusterLimit the given cluster limit.
   * @return the approximate network and the log base 2 of the maximum
   *   cluster size achieved.
   */
  
  public <T extends Scope> Result<T> constructApproximateScopes (
   Random r, Set<T> originalScopes, Map<Var,Var[]> origToClones,
   double logClusterLimit) throws Exception;

}
